import { __decorate, __metadata } from 'tslib';
import { NgIfContext, CommonModule } from '@angular/common';
import { ViewContainerRef, Input, Directive, NgModule } from '@angular/core';
import { AnimationFrameRef, GetComputedStyleRef, TimeoutRef } from 'ng-refs';

//
// Based off of the source/logic from the `utils.js` file of the Alpine.js project:
// https://github.com/alpinejs/alpine/blob/master/src/utils.js
//
function transitionClasses(animationFrameRef, getComputedStyleRef, timeoutRef, el, classesDuring, classesStart, classesEnd, done) {
    if (!el) {
        return;
    }
    transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, {
        start() {
            el.classList.add(...classesStart);
        },
        during() {
            el.classList.add(...classesDuring);
        },
        end() {
            el.classList.remove(...classesStart);
            el.classList.add(...classesEnd);
        },
        hide() {
            done();
        },
        cleanup() {
            el.classList.remove(...classesDuring);
            el.classList.remove(...classesEnd);
        }
    });
}
function transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, stages) {
    stages.start();
    stages.during();
    animationFrameRef.nativeRequest(() => {
        // Note: Safari's transitionDuration property will list out comma separated
        // transition durations for every single transition property. Let's grab the
        // first one and call it a day.
        const { transitionDuration } = getComputedStyleRef.native(el);
        const durationString = transitionDuration
            .replace(/,.*/, '')
            .replace('s', '');
        const duration = Number(durationString) * 1000;
        animationFrameRef.nativeRequest(() => {
            stages.end();
            timeoutRef.nativeSet(() => {
                stages.hide();
                // Adding an "isConnected" check, in case the callback
                // removed the element from the DOM.
                if (el.isConnected) {
                    stages.cleanup();
                }
            }, duration);
        });
    });
}

let NgTransitionDirective = class NgTransitionDirective {
    constructor(animationFrameRef, getComputedStyleRef, timeoutRef, _viewContainer) {
        this.animationFrameRef = animationFrameRef;
        this.getComputedStyleRef = getComputedStyleRef;
        this.timeoutRef = timeoutRef;
        this._viewContainer = _viewContainer;
        this._context = new NgIfContext();
        this._init = false;
        this._ngTransitionEnter = [];
        this._ngTransitionEnterStart = [];
        this._ngTransitionEnterEnd = [];
        this._ngTransitionLeave = [];
        this._ngTransitionLeaveStart = [];
        this._ngTransitionLeaveEnd = [];
    }
    ngOnInit() {
        this._init = true;
        this._updateView(true);
    }
    set ngTransition(condition) {
        this._context.$implicit = this._context.ngIf = condition;
        this._updateView();
    }
    set ngTransitionEnter(ngTransitionEnter) {
        this._ngTransitionEnter = ngTransitionEnter.split(' ');
    }
    set ngTransitionEnterStart(ngTransitionEnterStart) {
        this._ngTransitionEnterStart = ngTransitionEnterStart.split(' ');
    }
    set ngTransitionEnterEnd(ngTransitionEnterEnd) {
        this._ngTransitionEnterEnd = ngTransitionEnterEnd.split(' ');
    }
    set ngTransitionLeave(ngTransitionLeave) {
        this._ngTransitionLeave = ngTransitionLeave.split(' ');
    }
    set ngTransitionLeaveStart(ngTransitionLeaveStart) {
        this._ngTransitionLeaveStart = ngTransitionLeaveStart.split(' ');
    }
    set ngTransitionLeaveEnd(ngTransitionLeaveEnd) {
        this._ngTransitionLeaveEnd = ngTransitionLeaveEnd.split(' ');
    }
    _updateView(isFirst = false) {
        if (!this._init) {
            return;
        }
        const show = this._context.$implicit;
        if (isFirst) {
            this.setTargetElementDisplay(show ? '' : 'none');
        }
        else if (show) {
            this.show();
        }
        else {
            this.hide();
        }
    }
    show() {
        this.setTargetElementDisplay('');
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this.getTargetElement(), this._ngTransitionEnter, this._ngTransitionEnterStart, this._ngTransitionEnterEnd, () => void 0);
    }
    hide() {
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this.getTargetElement(), this._ngTransitionLeave, this._ngTransitionLeaveStart, this._ngTransitionLeaveEnd, () => this.setTargetElementDisplay('none'));
    }
    getTargetElement() {
        var _a, _b, _c;
        return (_c = (_b = (_a = this._viewContainer) === null || _a === void 0 ? void 0 : _a.element) === null || _b === void 0 ? void 0 : _b.nativeElement) !== null && _c !== void 0 ? _c : null;
    }
    setTargetElementDisplay(value) {
        const targetElement = this.getTargetElement();
        if (targetElement === null || targetElement === void 0 ? void 0 : targetElement.style) {
            targetElement.style.display = value;
        }
    }
};
NgTransitionDirective.ctorParameters = () => [
    { type: AnimationFrameRef },
    { type: GetComputedStyleRef },
    { type: TimeoutRef },
    { type: ViewContainerRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], NgTransitionDirective.prototype, "ngTransition", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnter", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnterStart", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnterEnd", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeave", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeaveStart", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeaveEnd", null);
NgTransitionDirective = __decorate([
    Directive({
        selector: '[ngTransition]'
    }),
    __metadata("design:paramtypes", [AnimationFrameRef,
        GetComputedStyleRef,
        TimeoutRef,
        ViewContainerRef])
], NgTransitionDirective);

let NgTransitionModule = class NgTransitionModule {
};
NgTransitionModule = __decorate([
    NgModule({
        imports: [CommonModule],
        declarations: [NgTransitionDirective],
        providers: [AnimationFrameRef, GetComputedStyleRef, TimeoutRef],
        exports: [NgTransitionDirective]
    })
], NgTransitionModule);

/**
 * Generated bundle index. Do not edit.
 */

export { NgTransitionDirective, NgTransitionModule };
//# sourceMappingURL=ng-transition.js.map
